/***************************************************************************
                          kmessage.cpp  -  description
                             -------------------
    begin                : Wed Mar 12 15:17:24 CET 2003
    copyright            : (C) 2003 by Jeroen Wijnhout
    email                : wijnhout@science.uva.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmessage.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qtimer.h>
#include <qfile.h>

#include <iostream>
            
KMessage::KMessage(const KMessageData *kmdata, QWidget *parent, const char *name) : QWidget(parent, name)

{
  int dw,dh;

  dbutton=0;

  //copy the KMessageData into kmd
  kmd = *kmdata;

  //get the message (we may need to read it from a file)
  retrieveMessage();

//
// LAYOUT
//  
  QVBoxLayout *layout = new QVBoxLayout(this);
  layout->setMargin(10);

  mes = new QTextEdit(kmd.message,NULL,this,"message");
  mes->setReadOnly(TRUE);
  mes->setTextFormat(QTextEdit::RichText);
  mes->setLinkUnderline(TRUE);
  mes->setWordWrap(QTextEdit::NoWrap);
  mes->setFont(qApp->font());

  //tricky
  //we set the minimum size for the message widget so that
  //the whole message is displayed (with a maximum size related to the desktop size)
  //in getReady(), which is called after the layout management is done
  //the minimum size is set to 10x10 so that the user can still make the
  //widget smaller
  QDesktopWidget *d = QApplication::desktop();
  dw = (int) (0.6* d->width()); if (kmd.width < dw) { dw = kmd.width + mes->verticalScrollBar()->width();}
  dh = (int) (0.6 *d->height()); if (kmd.height < dh ) { dh = kmd.height+ mes->horizontalScrollBar()->height(); }
  mes->setMinimumSize(dw,dh);
  

  //a QHBox to hold all the buttons            
  QHBox *bts = new QHBox(this);
  bts->setMargin(10);
  bts->setSpacing(10);
  
  //parse the button string
  KMessageButton *button;

  //if buttons are not specified add a default button  
  if (!kmd.butset) {
    button = new KMessageButton(DEFAULT_BUTTON,0,static_cast<QWidget*>(bts));

    //note that we connect clickedAt instead of clicked to handleButton	
    connect(button,SIGNAL(clickedAt(const QString &,int)),this,SLOT(handleButton(const QString &,int)));

    //since this is the only button, it is the default button
    dbutton = button;
  }
  else {
  //buttons are specified, create them all
  char c = ',';
  QString str;
  str = QString(kmd.buttons);
  str.append(c);
  int j=0,len=str.length(),nob=0;
  for (int i=0; i < len; i++) {
    if ( str.at(i) == ',') {
      button = new KMessageButton(str.mid(j,i-j),++nob,static_cast<QWidget*>(bts));
  
      //determine if this is the default button
      if (str.mid(j,i-j) == kmd.deflt) { dbutton = button; }     
 
      //note that we connect clickedAt instead of clicked to handleButton	
      connect(button,SIGNAL(clickedAt(const QString &,int)),this,SLOT(handleButton(const QString &,int)));
      j=i+1;
    }

  }
  } 
  
  
  layout->addWidget(mes);
  layout->addWidget(bts);

  //set the timer, if neccessary, that controls how long the widget is displayed
  if (kmd.timeout>0) {
    QTimer *timer = new QTimer();
    connect(timer,SIGNAL(timeout()),qApp,SLOT(quit()));
    timer->start(kmd.timeout*1000,true);
  }


}

KMessage::~KMessage()
{
}

void KMessage::polish() {

  //move the widget to the desired location
  //depending on whether -center or -nearmouse
  //were set
  int x,y,w,h,dw,dh;
  
  QDesktopWidget *d = QApplication::desktop();
  dw = d->width();
  dh = d->height();

  w=this->width();
  h=this->height(); 

  x=this->pos().x();
  y=this->pos().y();
  
     
  if (kmd.center) {
    x=(dw-w)/2;
    y=(dh-h)/2;
  }

  if (kmd.nearmouse) {
    x=this->cursor().pos().x();
    y=this->cursor().pos().y();
  }

  move(x,y); 

}

void KMessage::show() {
  QWidget::show();
  
  //minimum size for the message widget was set in the kmessage constructor
  //to fool the layout manager (otherwise the message widget was ALWAYS (100,100) pixels)
  //now set the minimum size back to something small so the user can make the window smaller
  //as he/she likes
  getReady();
}
  
void KMessage::retrieveMessage() {

  //get the size needed to display the message
  //size of the widget will be set accordingly
  kmd.width=fontMetrics().width(kmd.message);
  kmd.height=fontMetrics().height();

  //a file was specified to read the message from
  if (kmd.file != "" ) {

      //if a command line message was specified as well
      //add a newline to separate this message from the file message
      if (kmd.message != "") { kmd.message += '\n'; }
      
      QFile f;
      bool r;
      int rd;

      //file=-, read from standard input
      if (kmd.file == "-") {
        r=f.open(IO_ReadOnly,stdin);
      }
      else {
        f.setName(kmd.file);
        r=f.open(IO_ReadOnly);
      }

      if (r) {
        char buf[81];

        //read the file line by line
        while ((rd=f.readLine(buf,80))>0) {
          buf[rd]=0;
          kmd.message += QString(buf);

          //calculate the width and height of the text in pixels
          rd = fontMetrics().width(buf);
          if (rd>kmd.width) { kmd.width=rd;}
          kmd.height += fontMetrics().height()+fontMetrics().leading();
        }

        f.close();
      }
      else {
        std::cout << "could not open " << kmd.file << "\n";
      }
  }
 
}
  
void KMessage::getReady() {
  //set the minimum size to tiny (so the layout manager can make
  //the QTextEditor smaller than 100x100, I think there is a bug in
  //the layout management...)
  //and give focus to the default button (if specified)
  mes->setMinimumSize(10,10);
  if (dbutton) { dbutton->setFocus(); }
}
  


void KMessage::handleButton(const QString & button, int n) {
    //a button was clicked

    //we need to print the name of the button to stdout  
    if (kmd.print) {std::cout << button;}

    //also the return value of kmessage must depend on the number
    //of the button that was clicked
    if (n>0) { qApp->exit(100+n); }

    //bye bye!!!
    qApp->quit();
}



//
//KMessageButton implementation
//

//The philosophy is this:
//The button is clicked and sends a clicked() signal
//which is received by receive() (makes sense doesn't it) which
//in turn sends a clickedAt() signal which passes the button number and
//button text so that the end-receiver can determine which button was
//clicked from the number and text
//
//This is neccessary since we do not know how many buttons there are, so we
//cannot connect the buttons to standard slots as we do usually.
KMessageButton::KMessageButton(QString text,int n, QWidget *parent, const char *name)
 : KPushButton(text,parent,name)
{
  number=n;
  connect(this,SIGNAL(clicked()),SLOT(receive()));    
}

void KMessageButton::receive() {
  emit(clickedAt(this->text(),number));
}
