/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Wed Mar 12 15:17:24 CET 2003
    copyright            : (C) 2003 by Jeroen Wijnhout
    email                : wijnhout@science.uva.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>

#include "kmessage.h"

#include <iostream>

static const char *description =
	I18N_NOOP("KMessage is a small program that displays a message window\
   and a number of specified buttons.");

	
static KCmdLineOptions options[] =
{
  {"buttons <buttons>",I18N_NOOP("Comma separated list of buttons that should be displayed."),DEFAULT_BUTTON},
  {"file <file>",I18N_NOOP("Reads the message from <file>, - for stdin."),0},
  {"print",I18N_NOOP("Prints name of the button pressed to stdout."),0},
  {"center",I18N_NOOP("Centers the message window."),0},
  {"nearmouse",I18N_NOOP("Places the window near the mouse."),0},
  {"timeout <timeout>",I18N_NOOP("Shows the message for <timeout> seconds."),0},
  {"default <deflt>",I18N_NOOP("The default button."),DEFAULT_BUTTON},
  {"!+message",I18N_NOOP("The message that should be displayed."),0},
  { 0, 0, 0 }
};

int main(int argc, char *argv[])
{

  KAboutData aboutData( "kmessage", I18N_NOOP("KMessage"),
    VERSION, description, KAboutData::License_GPL,
    "(c) 2003, Jeroen Wijnhout", 0, 0, "wijnhout@science.uva.nl");
  aboutData.addAuthor("Jeroen Wijnhout",0, "wijnhout@science.uva.nl");
  KCmdLineArgs::init( argc, argv, &aboutData );
  KCmdLineArgs::addCmdLineOptions( options ); 

  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

  //
  //parse all the command line options
  //
  
  //holds all data necessary to show the message window
  //i.e. the message itself and all options specified on the command line
  KMessageData *kmd = new KMessageData;

  //a message is specified on the command line
  if (args->count() > 0) {
     for ( int i=0; i<args->count(); i++) {
       kmd->message +=args->arg(i);
       if (i>0) { kmd->message += '\n';}
     }
  } 

  //the message should be read from a file (possibly stdin)
  if (args->isSet("file")) {
    kmd->file = args->getOption("file");
  } 

  kmd->butset = args->isSet("buttons");
  kmd->buttons= args->getOption("buttons");
  kmd->deflt=args->getOption("default");
    
  if (args->isSet("timeout")) {kmd->timeout= (args->getOption("timeout")).toInt();}
  else { kmd->timeout = 0; }

  kmd->center=args->isSet("center");
  kmd->print=args->isSet("print");
  kmd->nearmouse=args->isSet("nearmouse");

  //create the message widget and pass the data (kmd) to the constructor
  KApplication a;
  KMessage *kmessage = new KMessage(kmd);
  a.setMainWidget(kmessage);

  kmessage->show();

  //kmessage->getReady();
    
  return a.exec();
}
